@@ -1,4 +1,4 @@
-#!/usr/bin/env python3.7
+#!/usr/bin/env python3
 """Render a tree from a predicate root pair.
 Normally run as a web service.
 
@@ -9,6 +9,7 @@ Usage:
 
 Options:
     -a --api=API            Full url to SciGraph api endpoint
+       --data-api=DAPI      Full url to SciGraph data api endpoint
     -k --key=APIKEY         apikey for SciGraph instance
     -p --port=PORT          port on which to run the server [default: 8000]
     -f --input-file=FILE    don't use SciGraph, load an individual file instead
@@ -52,7 +53,7 @@ from nifstd_tools.sheets_sparc import (hyperlink_tree,
                                        tag_row,
                                        open_custom_sparc_view_yml,
                                        YML_DELIMITER)
-from nifstd_tools.simplify import simplify, cleanBad
+from nifstd_tools.simplify import simplify, apinat_deblob, cleanBad
 from nifstd_tools import __version__
 
 log = makeSimpleLogger('ontree')
@@ -453,15 +454,19 @@ demo_examples = (
     ('Neuron connectivity', '/trees/sparc/demos/isan2019/neuron-connectivity'),
 )
 
-def server(api_key=None, verbose=False):
-    # data endpoint
-    #_dataBP = 'https://sparc.olympiangods.org/scigraph'
-    _dataBP = 'http://sparc-data.scicrunch.io:9000/scigraph'
-    scigraphd = moduleDirect(_dataBP, 'scigraphd')
+def server(api_key=None, verbose=False, data_endpoint=None):
+    if data_endpoint:
+        _dataBP = data_endpoint
+        scigraphd = moduleDirect(_dataBP, 'scigraphd')
+    else:
+        from pyontutils import scigraph as scigraphd
+
     data_sgd = scigraphd.Dynamic(cache=True, verbose=True, do_error=True)
-    data_sgd._basePath = _dataBP
     data_sgc = scigraphd.Cypher(cache=True, verbose=True)
-    data_sgc._basePath = _dataBP
+
+    if data_endpoint:
+        data_sgd._basePath = _dataBP
+        data_sgc._basePath = _dataBP
 
     f = Path(__file__).resolve()
     working_dir = get_working_dir(__file__)
@@ -711,7 +716,12 @@ def server(api_key=None, verbose=False):
 
     @app.route(f'/{basename}/sparc/simple/dynamic/<path:path>', methods=['GET'])
     def route_sparc_simple_dynamic(path):
-        return sparc_dynamic(data_sgd, data_sgc, path, wgb, simplify)
+        if '/neru-' in path:
+            process = lambda x, blob: apinat_deblob(blob, remove_converge=True)[0]
+        else:
+            process = simplify
+
+        return sparc_dynamic(data_sgd, data_sgc, path, wgb, process)
 
     @app.route(f'/{basename}/sparc/dynamic/<path:path>', methods=['GET'])
     def route_sparc_dynamic(path):
@@ -1111,7 +1121,12 @@ def main():
             scs.api_key = api_key
             scs.setup(instrumented=OntTerm)
 
-        app = server(verbose=verbose)
+        _data_endpoint = args['--data-api']
+        data_endpoint = (
+            _data_endpoint if _data_endpoint else
+            scigraph.scigraph_client.BASEPATH)
+
+        app = server(verbose=verbose, data_endpoint=data_endpoint)
         # app.debug = False
         # app.run(host='localhost', port=args['--port'], threaded=True)  # nginxwoo
         # app.debug = True
